import pickle
from collections import UserDict
from colorama import Fore


class Notebook(UserDict):
    _instance = None

    def __new__(cls, *args, **kwargs):

        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    file_name = 'Notebook.bin'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.load_notes()

    def show_all_records(self):
        return self.data

    def iterate(self, n=1):
        for key, _ in self.data.items():
            d_list = list(self.data.values())
            for i in range(0, len(d_list), n):
                yield key, d_list[i:i + n]

    def add_record(self, record):
        self.data[record.title.value] = record

    def save_notes(self):
        with open(self.file_name, 'wb') as file:
            pickle.dump(self.data, file)
        print(f'Your notes are saved!')

    def load_notes(self):
        try:
            with open(self.file_name, 'rb') as file:
                self.data = pickle.load(file)
        except:
            return


class Record:
    def __init__(self, title=None, text=None, tag=None):
        self.title = title
        self.text = text
        self.tags = []
        if tag:
            self.tags.append(tag)

    def add_tag(self, tag):
        self.tags.append(tag)

    def create_tag(self, record, user_tag=None, update=False):
        if user_tag:
            for _ in range(10):
                tag = Tag(user_tag)
                if update:
                    record.tags = [tag]
                else:
                    record.add_tag(tag)
                    break

    def create_text(self, record, user_text):
        if user_text:
            for _ in range(10):
                text = Text(user_text)
                record.text = text
                break

    def create_title(self, record, user_title):
        if user_title:
            for _ in range(10):
                title = Title(user_title)
                record.title = title
                break

    def formatting_record(self, record):
        title = getattr(record, 'title', '')
        if title:
            title_value = title.value
        else:
            title_value = "not found"

        text = getattr(record, 'text', '')
        if text:
            text_value = text.value
        else:
            text_value = "not found"

        tags = getattr(record, 'tags', '')
        if tags:
            tag_value = [tag.value for tag in tags]
        else:
            tag_value = "not found"

        return {"title": title_value, "text": text_value, "#tag": tag_value}


class Field:
    def __init__(self, value):
        self._value = value

    @property
    def value(self):
        return self._value


class Title(Field):
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        if len(value) >= 20:
            raise ValueError


class Text(Field):
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        if len(value) >= 150:
            raise ValueError


class Tag(Field):
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        if len(value) >= 5:
            raise ValueError


def main():
    notebook = Notebook()
    print(Fore.LIGHTBLUE_EX + '-' * 52)
    print('|You can use following commands:\n'
          '|add - add a new note in Notebook\n'
          '|del - delete a note from Notebook\n'
          '|change - change a note in Notebook\n'
          '|find - find note in Notebook\n'
          '|tag sort - sorts notes by tags in Notebook\n'
          '|show all - shows the entire Notebook\n'
          '|close, exit, goodbye or . - closing the program\n')
    print('-' * 52)
    while True:
        user_inp = input('Enter command: ').lower().strip()
        user_exit_list = ['goodbye', 'close', 'exit', '.']
        if user_inp in user_exit_list:
            print('Goodbye!\n'
                  'Your Notebook has been successfully saved!')
            break
        elif user_inp == 'hello':
            print('How can I help you?')
            continue
        elif 'add' in user_inp:
            CommandsHandler.add_note(notebook)
        elif 'del' in user_inp:
            CommandsHandler.remove_note(notebook)
        elif 'change' in user_inp:
            CommandsHandler.change_note(notebook)
        elif 'find' in user_inp:
            CommandsHandler.find_note(notebook)
        elif 'tag sort' in user_inp:
            CommandsHandler.sort_notes_by_tag(notebook)
        elif 'show all' in user_inp:
            CommandsHandler.show_all_notes(notebook)
        else:
            print('Choose the right command!')
            continue


class CommandsHandler:
    notebook = Notebook()

    def add_note(self):
        user_title = input("Enter a title: ")
        title = Title(user_title)
        #record.title = title
        record = Record(title=title)
        record.create_title(record=record, user_title=user_title)
        user_text = input("Enter a text of note: ")
        text = Text(user_text)
        record.text = text
        record.create_text(record=record, user_text=user_text)
        user_tag = input("Enter a #tag: ")
        tag = Tag(user_tag)
        record.tag = tag
        record.create_tag(record=record, user_tag=user_tag)
        self.notebook.add_record(record)
        self.notebook.save_notes()

    def show_all_notes(self):
        data = self.notebook.show_all_records()
        if not data:
            print('The notebook is empty.')
        else:
            for title, record in data.items():
                rec_data = record.formatting_record(record)
                print(f"|Title: {title}\n"
                      f"|Text: {rec_data['text']}\n"
                      f"|Tag: {rec_data['#tag']}\n")

    def find_note(self):
        find_user = input('Enter title or #tag: ')
        data = self.notebook.show_all_records()
        if not data:
            print('The notebook is empty.')
        else:
            flag = False
            for title, record in data.items():
                rec_data = record.formatting_record(record)
                if title.startswith(find_user):
                    flag = True
                    print(f"|Title: {title}\n"
                          f"|Text: {rec_data['text']}\n"
                          f"|Tag: {rec_data['#tag']}\n")
                tag = getattr(record, 'tag', '')
                if tag:
                    if tag.value.startswith(find_user):
                        flag = True
                        print(f"|Title: {title}\n"
                              f"|Text: {rec_data['text']}\n"
                              f"|Tag: {rec_data['#tag']}\n")
            if not flag:
                print('Note with this title or #tag was not found.')

    def sort_notes_by_tag(notebook):
        #notebook.notes = sorted(notebook.notes, key=lambda x: x.tags)
        pass

    def change_note(self):
        change_user = input('Enter title of note: ')
        data = self.notebook.show_all_records()
        if not data:
            print('The Notebook is empty.')
        else:
            flag = False
            for title, record in data.items():
                rec_data = record.formatting_record(record)
                if title.startswith(change_user):
                    flag = True
                    print("-"*50)
                    print(f"|add tag - press 1|\n"
                          f"|change title - press 2|\n"
                          f"|change text - press 3|\n"
                          f"|change tag - press 4|")
                    print("-" * 50)
                    change = int(input('Enter your choice: '))
                    if change == 1:
                        tag_add = input('Enter a tag: ')
                        record.create_tag(record=record, user_tag=tag_add,
                                          update=False)
                        print(f'In note {title} append '
                              f'{[tag.value for tag in record.tags]}')
                    elif change == 2:
                        new_title = input('Enter a new title: ')
                        record.title = Title(new_title)
                        print(f'In note title {title} was changed to '
                              f'{record.title.value}')
                    elif change == 3:
                        text = input('Enter a new text: ')
                        record.create_text(
                            record=record, user_text=text)
                        print(f'In note {title} change text '
                              f'{record.text.value}')
                    elif change == 4:
                        tag_add = input('Enter a tag: ')
                        record.create_tag(record=record, user_tag=tag_add,
                                          update=True)
                        print(f'In note {title} update '
                              f'{[tag.value for tag in record.tags]}')
                    else:
                        print(f'{change} invalid choice')
                        return
            self.notebook.save_notes()

    def remove_note(self):
        print('|del - Delete note|\n'
              '|del all - Clean Noteook|')
        remove_date = input('Enter your choice: ')
        if remove_date == 'del':
            remove_note = input('Enter a title of the note to be deleted: ')
            self.notebook.data.pop(remove_note)
            print(f'Note {remove_note} deleted.')
        elif remove_date == 'del all':
            print(f'Are you sure you want to clear the Notebook?')
            question = input('Y or N: ').lower().strip()
            if question == 'n':
                print('non')
                return
            elif question == 'y':
                print('lol')
                self.notebook.data.clear()
        self.notebook.save_notes()


# ------------------------------------------------ADAPTER-------------------------------------------------------
help = ('|You can use following commands:\n'
        '|add - add a new note in Notebook\n'
        '|del - delete a note from Notebook\n'
        '|change - change a note in Notebook\n'
        '|find - find note in Notebook\n'
        '|tag sort - sorts notes by tags in Notebook\n'
        '|show all - shows the entire Notebook\n'
        '|back - Closing the sublayer\n')

commands = {'add': CommandsHandler().add_note,
            'del': CommandsHandler().remove_note,
            'change': CommandsHandler().change_note,
            'find': CommandsHandler().find_note,
            'tag sort': CommandsHandler().sort_notes_by_tag,
            'show all': CommandsHandler().show_all_notes,
            'back': ...}

CONFIG = ({'help': help,
           'commands': commands
           })


if __name__ == "__main__":
    main()



import difflib


class Handler:
    def __init__(self, help, commands, database=None):
        self.database = database
        self.commands = commands
        self.help = help

    def get_command_suggestion(self, query):
        return difflib.get_close_matches(query, self.commands.keys(), n=1, cutoff=0.6)
    
    def execute_command(self, query):
        if self.database:
            self.commands[query](self.database)
        self.commands[query]()
    
    def run(self):
        print(self.help)
        while True:
            query = input('> ')
            if query == 'back':
                break
            try:
                self.execute_command(query)
            except KeyError:
                suggestion = self.get_command_suggestion(query)
                if suggestion:
                    print(f'Did you mean {suggestion[0]}?')
                else:
                    print('Invalid command')
                    print(self.help)



from setuptools import setup, find_namespace_packages


setup(
    name='GTTD',
    version='0.1.15',
    description='Console bot organizer',
    url='https://github.com/C-Hurtmann/Going_to_the_Dream',
    license='MIT',
    packages=find_namespace_packages(include=['bot.*', 'bot']),
    install_requires=['inquirer', 'colorama', 'prettytable'],
    entry_points={'console_scripts': ['helper = bot.interface:main']}
)



import inquirer

from bot.sublayers.address_book import CONFIG as ab_config
from bot.sublayers.notebook import CONFIG as nb_config
from bot.sublayers.cleaner import CONFIG as sa_config
from bot.sublayers.handler import Handler


main_menu = [
    inquirer.List('option',
                  message='You are in main menu. Please select an option:',
                  choices=[
                      'Address Book',
                      'Note Book',
                      'Sorter Assist',
                      'Exit'
                  ])
]


def main():
    while True:
        main_choice = inquirer.prompt(main_menu)['option']

        if main_choice == 'Address Book':
            handler = Handler(**ab_config)
            handler.run()

        elif main_choice == 'Note Book':
            handler = Handler(**nb_config)
            handler.run()

        elif main_choice == 'Sorter Assist':
            handler = Handler(**sa_config)
            handler.run()
        elif main_choice == 'Exit':
            print("Exiting program...")
            break


if __name__ == '__main__':
    main()



import pickle
from collections import UserDict
from datetime import datetime, timedelta
import re
from colorama import init, Fore
from colorama import Back
from colorama import Style
from prettytable import PrettyTable
init(autoreset=True)


class AddressBook(UserDict):
    _instance = None

    def __new__(cls, *args, **kwargs):

        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    file_name = 'AddressBook.bin'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.load_contacts()

    def show_all_records(self):
        return self.data

    def iterate(self, n=1):
        for key, value in self.data.items():
            d_list = list(self.data.values())
            for i in range(0, len(d_list), n):
                yield key, d_list[i:i + n]

    def add_record(self, record):
        self.data[record.name.value] = record

    def save_contacts(self):
        with open(self.file_name, 'wb') as f:
            pickle.dump(self.data, f)
        print(Style.BRIGHT+Fore.YELLOW+f'Your contact saved!')

    def load_contacts(self):
        try:
            with open(self.file_name, 'rb') as f:
                self.data = pickle.load(f)
        except:
            return


class Record:
    def __init__(self, name, phone=None, email=None, birthday=None,
                 home_address=None):
        self.name = name
        self.email = email
        self.birthday = birthday
        self.phones = []
        self.home_address = home_address
        if phone:
            self.phones.append(phone)


    def add_phone(self, phone):
        self.phones.append(phone)


    def create_phone(self, record, user_input=None, update=False):
        if user_input:
            for i in range(10):
                phone = Phone(user_input)
                if phone.validate_phone(user_input):
                    if update:
                        record.phones = [phone]
                    else:
                        record.add_phone(phone)
                    break
                else:
                    print(Style.BRIGHT+Fore.RED+"Incorrect phone number "
                                                "format entered.\n"
                          "Enter your phone in the format '+380991122333'")
                    user_input = input(Style.BRIGHT+Fore.BLUE +
                                       "Enter contact phone: ")

    def create_email(self, record, user_email):
        if user_email:
            for i in range(10):
                email = Email(user_email)
                if email.validate_email(user_email):
                    record.email = email
                    break
                else:
                    print(Style.BRIGHT+Fore.RED+"Email entered incorrectly.\n"
                          "Please enter a valid email: 'example@gmail.com'")
                    user_email = input(Style.BRIGHT+Fore.BLUE +
                                       "Enter contact email: ")

    def create_birthday(self, record, user_birthday):
        if user_birthday:
            for i in range(10):
                birthday = Birthday(user_birthday)
                if birthday.validate_birthday(user_birthday):
                    record.birthday = birthday
                    break
                else:
                    print(Style.BRIGHT+Fore.RED+"Birthday invalid.\n"
                          "Birthday should be in the format\n"
                          "'day.month.year' and less than current date.")
                    user_birthday = input(Style.BRIGHT+Fore.BLUE +
                                          "Enter contact Birthday: ")

    def home_address_create(self, record, user_address):
        if user_address:
            home_address = HomeAddress(user_address)
            record.home_address = home_address

    def formatting_record(self, record):
        phones = getattr(record, 'phones', '')
        if phones:
            p_l = [phone.value for phone in phones]
            phone_val = p_l
        else:
            phone_val = "Phone number missing."
        email = getattr(record, 'email', '')
        if email:
            email_val = email.value
        else:
            email_val = "Email is missing."
        birthday = getattr(record, 'birthday', '')
        if birthday:
            birthday_val = birthday.value
        else:
            birthday_val = "Date of birth is missing."
        home_address = getattr(record, 'home_address', '')
        if home_address:
            home_address_val = home_address.value
        else:
            home_address_val = "Home address is missing."

        return {"phone": phone_val, "email": email_val,
                "birthday": birthday_val, "home_address": home_address_val}



class Field:
    def __init__(self, value):
        self._value = value

    @property
    def value(self):
        return self._value


class Name(Field):
    pass


class Phone(Field):
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        if not value.startswith('+'):
            raise ValueError
        if len(value) != 13:
            raise ValueError

    def validate_phone(self, phone):
        pattern = r"^[\+]?3?[\s]?8?[\s]?\(?0\d{2}?\)?" \
                  r"[\s]?\d{3}[\s|-]?\d{2}[\s|-]?\d{2}$"
        a = re.match(pattern, phone)
        if a is not None:
            return phone


class Email(Field):

    def validate_email(self, email):
        pattern = r"^[-\w\.]+@([-\w]+\.)+[-\w]{2,4}$"
        a = re.match(pattern, email)
        if a is not None:
            return email


class Birthday(Field):
    @property
    def value(self):
        return self._value

    @value.setter
    def set_value(self, value):
        try:
            datetime.strptime(value, '%d.%m.%Y')
        except:
            raise ValueError

    def validate_birthday(self, birthday):
        try:
            bt_obj = datetime.strptime(birthday, '%d.%m.%Y')
            if bt_obj.date() >= datetime.now().date():
                return None
            return birthday
        except:
            pass


class HomeAddress(Field):
    pass


class CommandsHandler:
    address_book = AddressBook()

    def add_contacts(self):
        user_name = input(Style.BRIGHT+Fore.BLUE + "Enter contact name: ")
        if not user_name:
            print("\033[4m\033[31m\033[45m{}\033[0m".format
                  ("Contact name is required"))
            return
        else:
            name = Name(user_name)
        record = Record(name)

        user_phone = input(Style.BRIGHT+Fore.BLUE + "Enter contact phone: ")
        record.create_phone(record=record, user_input=user_phone, update=True)

        user_email = input(Style.BRIGHT+Fore.BLUE + "Enter contact email: ")
        record.create_email(record=record, user_email=user_email)

        user_birthday = input(Style.BRIGHT+Fore.BLUE +
                              "Enter contact Birthday: ")
        record.create_birthday(record=record, user_birthday=user_birthday)

        user_home_address = input(Style.BRIGHT+Fore.BLUE +
                                  "Enter contact home address: ")
        record.home_address_create(record=record,
                                   user_address=user_home_address)
        self.address_book.add_record(record)
        self.address_book.save_contacts()

    def show_all_contacts(self):
        data = self.address_book.show_all_records()
        if not data:
            print("\033[4m\033[31m\033[45m{}\033[0m".format
                  ('The address book is empty.'))
        else:
            for name, record in data.items():
                rec_data = record.formatting_record(record)
                print(Fore.GREEN+f"|Name: {name}, Phone: {rec_data['phone']}, "
                      f"Email: {rec_data['email']}, "
                      f"Birthday: {rec_data['birthday']},"
                      f"Home address: {rec_data['home_address']}|")

    def find_contacts(self):
        find_user = input(Style.BRIGHT+Fore.BLUE +
                          'Enter contact name or phone: ')
        data = self.address_book.show_all_records()
        if not data:
            print("\033[4m\033[31m\033[45m{}\033[0m".format
                  ('The address book is empty.'))
        else:
            flag = False
            for name, record in data.items():
                rec_data = record.formatting_record(record)
                if name.startswith(find_user):
                    flag = True
                    find_list = f"|Name: {name}, Phone: {rec_data['phone']}," \
                                f"Email: {rec_data['email']}," \
                                f"Birthday: {rec_data['birthday']}," \
                                f"Home address: {rec_data['home_address']}|"
                    print("\033[1m\033[35m{}\033[0m".format(find_list))
                phones = getattr(record, 'phones', '')

                if phones:
                    for phone in phones:
                        if phone.value.startswith(find_user):
                            flag = True
                            print(f"Name: {name}, Phone: {rec_data['phone']}, "
                                  f"Email: {rec_data['email']}, "
                                  f"Birthday: {rec_data['birthday']}")
            if not flag:
                print("\033[4m\033[31m\033[45m{}\033[0m".format
                      ('Contact with this name or phone number was '
                       'not found.'))

    def birthday_contacts(self):
        birth_user = int(input(Style.BRIGHT+Fore.BLUE +
                               'Enter a number of days: '))
        flag = False
        now = datetime.now().date()
        data = self.address_book.show_all_records()
        current_date = now + timedelta(days=birth_user)
        for name, record in data.items():
            rec_data = record.formatting_record(record)
            if record.birthday:
                birth = rec_data['birthday']
                new_user_date = datetime.strptime(birth, "%d.%m.%Y").date()
                new_date = datetime(day=new_user_date.day,
                                    month=new_user_date.month,
                                    year=now.year).date()
                if new_date >= now and new_date < current_date:
                    flag = True
                    print(f"Name: {name}, Phone: {rec_data['phone']}, "
                          f"Email: {rec_data['email']}, "
                          f"Birthday: {rec_data['birthday']}")
        if not flag:
            print("\033[4m\033[31m\033[45m{}\033[0m".format
                  ('There are no birthdays in this range!'))


    def change_contacts(self):
        change_user = input(Style.BRIGHT+Fore.CYAN + 'Enter contact name: ')
        data = self.address_book.show_all_records()
        if not data:
            print("\033[4m\033[31m\033[45m{}\033[0m".format
                  ('The address book is empty.'))
        else:
            flag = False
            update_name_data = {}
            for name, record in data.items():
                rec_data = record.formatting_record(record)
                if name.startswith(change_user):
                    flag = True
                    change_commands = PrettyTable()
                    change_commands.field_names = \
                        ["Command entry", "Command value"]
                    change_commands.add_row(
                        ["Press 1", "Add a phone number to a contact"])
                    change_commands.add_row(
                        ["Press 2", "Change contact email"])
                    change_commands.add_row(
                        ["Press 3", "Change contact birthday"])
                    change_commands.add_row(
                        ["Press 4", "Change contact name"])
                    change_commands.add_row(
                        ["Press 5", "Change contact phone number"])
                    change_commands.add_row(
                        ["Press 6", "Change contact home address"])
                    print("\033[1m\033[36m{}\033[0m".format(change_commands))
                    change = input(Style.BRIGHT+Fore.CYAN +
                                       'Enter your choice: ')
                    if change == '1':
                        num = input(Style.BRIGHT+Fore.CYAN + 'Enter number: ')
                        record.create_phone(record=record, user_input=num,
                                            update=False)
                        print(Style.BRIGHT + Back.BLUE + Fore.RED +
                              f'In contact {name} append '
                              f'{[phone.value for phone in record.phones]}')
                    elif change == '2':
                        mail = input(Style.BRIGHT+Fore.CYAN +
                                     'Enter new email: ')
                        record.create_email(record=record, user_email=mail)
                        print(Back.BLUE + Fore.RED +
                              f'In contact {name} change or append email '
                              f'{record.email.value}')
                    elif change == '3':
                        birthday = input(Style.BRIGHT+Fore.CYAN +
                                         'Enter new date: ')
                        record.create_birthday(record=record,
                                               user_birthday=birthday)
                        print(
                            Style.BRIGHT + Back.BLUE + Fore.RED +
                            f'In contact {name} change or append date birthday'
                            f'{record.birthday.value}')
                    elif change == '4':
                        new_name = input(Style.BRIGHT+Fore.CYAN +
                                         'Enter new name: ')
                        record.name = Name(new_name)
                        update_name_data[name] = new_name
                        print("\033[3m\033[33m\033[41m{}\033[0m".format
                              ('Contact name changed to:'), Style.BRIGHT +
                              Fore.LIGHTGREEN_EX + record.name.value)
                    elif change == '5':
                        num = input(Style.BRIGHT+Fore.CYAN +
                                    'Enter number: ')
                        record.create_phone(record=record, user_input=num,
                                            update=True)
                        print(Style.BRIGHT + Back.BLUE + Fore.RED +
                              f'In contact {name} update '
                              f'{[phone.value for phone in record.phones]}')
                    elif change == '6':
                        new_address = input(Style.BRIGHT+Fore.CYAN +
                                            'Enter new address: ')
                        record.home_address_create(record=record,
                                               user_address=new_address)
                        print(
                            Style.BRIGHT + Back.BLUE + Fore.RED +
                            f'In contact {name} change or append home address'
                            f' {record.home_address.value}')
                    else:
                        print(Style.BRIGHT+Fore.RED +
                              f'{change} invalid choice')
                        return
            for name, new_name in update_name_data.items():
                self.address_book.data[new_name] = \
                    self.address_book.data.pop(name)
            if flag:
                self.address_book.save_contacts()
            else:
                print("\033[4m\033[31m\033[45m{}\033[0m".format
                      ("User not fount"))

    def remove_contacts(self):
        remove_commands = PrettyTable()
        remove_commands.field_names = ["Command entry", "Command value"]
        remove_commands.add_row(["del", "Delete one selected contact"])
        remove_commands.add_row(["del all",
                                 "Delete all address book contacts"])
        print("\033[1m\033[31m{}\033[0m".format(remove_commands))
        remove_date = input(Style.BRIGHT+Fore.YELLOW + 'Enter your choice: ')
        if remove_date == 'del':
            remove_user = input(Style.BRIGHT+Fore.YELLOW +
                                'Enter the name of the contact '
                                'to be deleted: ')
            self.address_book.data.pop(remove_user)
            print(Style.BRIGHT+Fore.RED + f'Contact {remove_user} deleted.')
            self.address_book.save_contacts()
        elif remove_date == 'del all':
            print(Style.BRIGHT+Fore.RED+f'Are you sure you want to '
                                        f'clear the Address Book?')
            question = input(Style.BRIGHT+Fore.RED+'Y or N: ').lower().strip()
            if question == 'n':
                return
            elif question == 'y':
                self.address_book.data.clear()
            self.address_book.save_contacts()
        else:
            print(Style.BRIGHT + Fore.RED + f'Invalid command')


# ------------------------------------------------ADAPTER-------------------------------------------------------

help = ('|You can use following commands:\n'
          '|add - Add new contact\n'
          '|find - Find contact in Address Book\n'
          '|show all - Shows the entire Address Book\n'
          '|get bith - Show birthdays\n'
          '|change - Change contact\n'
          '|del - Delete contact from address book\n'
          '|back - Closing the sublayer\n')

commands = {'add': CommandsHandler().add_contacts,
            'find': CommandsHandler().find_contacts,
            'show all': CommandsHandler().show_all_contacts,
            'get bith': CommandsHandler().birthday_contacts,
            'change': CommandsHandler().change_contacts,
            'del': CommandsHandler().remove_contacts,
            'back': ...}

CONFIG = ({'help': help,
           'commands': commands})





import os
import shutil
from pathlib import Path
from colorama import init, Fore

init(autoreset=True)

extensions = {'video': ['mp4', 'mov', 'avi', 'mkv'],
              'audio': ['mp3', 'wav', 'ogg', 'amr'],
              'images': ['jpg', 'png', 'jpeg', 'svg'],
              'archives': ['zip', 'gz', 'tar'],
              'documents': ['pdf', 'txt', 'doc', 'docx', 'xlsx', 'pptx', 'odt'],
              'others': []
}

CYRILLIC_SYMBOLS = "абвгдеёжзийклмнопрстуфхцчшщъыьэюяєіїґ#$%&()^+-:;<=>?@[\]{|`~}!"
TRANSLATION = ("a", "b", "v", "g", "d", "e", "e", "j", "z", "i", "j", "k", "l", "m", "n", "o", "p", "r", "s", "t", "u",
               "f", "h", "ts", "ch", "sh", "sch", "", "y", "", "e", "yu", "ya", "je", "i", "ji", "g", "_", "_", "_", "_", "_", "_",
               "_", "_", "_", "_", "_", "_", "_", "_", "_", "_", "_", "_", "_", "_", "_", "_", "_", "_","_")

TRANS = {}

for c, l in zip(CYRILLIC_SYMBOLS, TRANSLATION):
    TRANS[ord(c)] = l
    TRANS[ord(c.upper())] = l.upper()

def normalize(name):
    name = Path(name).name 

    new_name = name.translate(TRANS)
    return new_name 

def create_folders_from_list(folder_path, folder_names):
    for folder in folder_names:
        try:

            if not os.path.exists(f'{folder_path}/{folder}'):
                os.mkdir(f'{folder_path}/{folder}')

        except FileExistsError:
            pass


file_paths = []
subfolder_paths = []

def paths (path, level = 1):

    names_dir = os.listdir(path) 
    
    file_paths.extend ([f.path for f in os.scandir(path) if not f.is_dir()])

    subfolder_paths.extend ([f.path for f in os.scandir(path) if f.is_dir()])
    for elem in names_dir:

        if os.path.isdir(path + "/" + elem):
            paths (path + "/" + elem, level + 1)

           
    return file_paths, subfolder_paths

    


def sort_files(path):  # TODO create folder for files with unknown extention
    
    ext_list = list(extensions.items())
    
    for file_path in file_paths:
        file_path = str(file_path)
        extension = file_path.split('.')[-1]

        file_name = file_path.split('/')[-1]

       
        for dict_key_int in range(len(ext_list)):
            
            if extension in ext_list[dict_key_int][1]:


                shutil.move(file_path, f'{path}/{ext_list[dict_key_int][0]}/{normalize(file_name)}')
                
    for ar_file in os.listdir(path + "/" + "archives"):
        try:
            shutil.unpack_archive(path + "/" + "archives" + "/" + ar_file, path + "/" + "archives")
            os.remove(path + "/" + "archives" + "/" + ar_file)

        except shutil.ReadError:
            pass
        
       
    names_file = [name for name in os.listdir(path) if os.path.isfile(os.path.join(path,name))]
    for unkn_file in names_file:

        
            shutil.move(path + "/" + unkn_file, path + "/" + "others" + "/" + normalize(unkn_file)) 

def remove_empty_folders(main_path, level = 1):
    for p in subfolder_paths:
        p = str (p)
        if not os.listdir(p):
            try:
                os.rmdir(p)
                remove_empty_folders(main_path + "/" + p, level + 1)    
            except FileNotFoundError:
                pass



def sort():
    
    try: 
        main_path = input(Fore.MAGENTA + "Enter path for folder: ")    
        create_folders_from_list(main_path, extensions)
        paths (main_path)
        sort_files(main_path)
        remove_empty_folders (main_path)
        print (Fore.MAGENTA + "Your files are sorted.\n" + "Deleting empty folders")
        for name_dir in os.listdir(main_path):
            print()
            print (Fore.MAGENTA + f"{name_dir.capitalize()}: ")
            for name_fale in os.listdir(main_path + "/" + name_dir):
                print (Fore.CYAN + f"    - {name_fale}")
    
    except FileNotFoundError:
            print (Fore.LIGHTRED_EX + "The path was wroning. Try again")


# ------------------------------------------------ADAPTER-------------------------------------------------------

help = ('|You can use following commands:\n'
          '|sort - Sorting the folder\n'
          '|back - Closing the sublayer\n')

commands = {'sort': sort,
            'back': ...}



CONFIG = ({'help': help,
           'commands': commands})

 
if __name__ == "__main__": 
    sort()
